from time import sleep

import requests
from elasticsearch_dsl import UpdateByQuery


class ElasticsearchFunctions:

    def __init__(self, config):
        self.config = config
        self.session = requests.Session()

    @staticmethod
    def check_or_create_index(esc, index_name, settings):
        response = esc.indices.exists(index_name)
        if response is True:
            return "EXISTED"
        else:
            esc.indices.create(index_name, body=settings)
            return "CREATED"

    def create_index_pattern(self, dest_es_url, index_patter_name, namespace, date_field):
        headers = {'Accept': '*/*', 'kbn-xsrf': 'true', 'Content-Type': 'application/json'}
        data = f'{{"attributes":{{"title":"{index_patter_name}","timeFieldName":"{date_field}"}}}}'
        self.session.auth = (self.config.ES_USER, self.config.ES_PASSWORD)

        post_url = f"{dest_es_url}{f'/s/{namespace}' if namespace else ''}/api/saved_objects/index-pattern"

        response = self.session.post(post_url, headers=headers, data=data)

        if response.status_code == 200:
            return True
        else:
            print(f"error creating index {index_patter_name} on {dest_es_url}")
            return False

    def create_space(self, url, namespace):
        headers = {'kbn-xsrf': 'true', 'Content-Type': 'application/json'}
        data = f'{{"id": "{namespace}", "name": "{namespace}", "initials": "{namespace.upper()[0:2]}", ' \
               f'"disabledFeatures":[]}}'
        self.session.auth = (self.config.ES_USER, self.config.ES_PASSWORD)
        response = self.session.post(f'{url}/api/spaces/space', headers=headers, data=data)
        return response == 200

    def get_object_id(self, kib_url, namespace, type_str, name):

        url = f"{kib_url}{f'/s/{namespace}' if namespace else ''}/api/saved_objects/_find"

        params = (
            ('type', type_str),
            ('search_fields', 'title'),
            ('search', name),
        )
        self.session.auth = (self.config.ES_USER, self.config.ES_PASSWORD)
        response = self.session.get(url, params=params, verify=False)
        if response.status_code != 200:
            print(f"{response.status_code}, {self.config.ES_USER}, {self.config.ES_PASSWORD}, {url}")
            return None

        response_json = response.json()
        if 'total' in response_json and response_json['total'] > 0:
            return response_json['saved_objects'][0]['id']
        else:
            return None

    def get_index_pattern(self, url, namespace, id_str):
        headers = {
            'Accept': '*/*',
            'Content-Type': 'application/json'
        }
        data = f'[{{"id":"{str(id_str)}","type":"index-pattern"}}]'
        self.session.auth = (self.config.ES_USER, self.config.ES_PASSWORD)

        bulk_url = f"{url}{f'/s/{namespace}' if namespace else ''}/api/saved_objects/_bulk_get"

        response = self.session.post(bulk_url, headers=headers, data=data)
        return response.json()

    @staticmethod
    def check_index_not_empty(index, esc):
        empty_query = {}
        resav = esc.search(index=index, body=empty_query)
        return len(resav['hits']['hits']) > 0

    @staticmethod
    def delete_from_day(esc, index, from_date, to_date):

        query = '''{
                      "query": {
                            "range" : {
                                "updated" : {
                                    "gte" : "%s",
                                    "lte" : "%s"
                                }
                            }
                        }
                    }''' % (from_date, to_date)
        result = esc.delete_by_query(index, query)
        return result

    @staticmethod
    def delete_index(esc, index):
        esc.indices.delete(index=index, ignore=[400, 404])

    @staticmethod
    def run_update_by_query(self, query, index):

        ubq = UpdateByQuery(using=self.config.es, index=index).update_from_dict(
            query).params(request_timeout=100)
        finished = False
        count = 0
        while not finished and count < 3:
            try:
                count += 1
                ubq.execute()
                finished = True
            except Exception as e:
                print(e)
                sleep(10 * count)
                pass


import os
from elasticsearch import Elasticsearch
from etlelk.settings_generic import body_settings_generic


class ConfigBase:
    KIBANA_SAVED_OBJECTS_PATH = os.environ.get('KIBANA_SAVED_OBJECTS_PATH', ".")
    KIBANA_HOST = os.environ.get('ES_HOST', 'localhost')
    KIBANA_PORT = os.environ.get('ES_PORT', '5601')
    ES_HOST = os.environ.get('ES_HOST', 'localhost')
    ES_PORT = os.environ.get('ES_PORT', '9200')
    ES_USER = os.environ.get('ES_USER', 'admin')
    ES_PASSWORD = os.environ.get('ES_PASSWORD', 'pass')
    ES_USE_SSL = os.environ.get('ES_USE_SSL') == "True"
    ES_SERVICOS_INDEX = os.environ.get('ES_SERVICOS_INDEX', 'servicos')
    ES_SOURCECODE_INDEX = os.environ.get('ES_SOURCECODE_INDEX', 'sourcecode')
    ES_TAGCLOUD_INDEX = os.environ.get('ES_TAGCLOUD_INDEX', 'tagcloud')

    es_kb_protocol = 'https' if ES_USE_SSL else 'http'

    ES_URL = f"{es_kb_protocol}://{ES_HOST}:{ES_PORT}"
    KIBANA_URL = f"{es_kb_protocol}://{KIBANA_HOST}:{KIBANA_PORT}"
    
    es = Elasticsearch(
        hosts=[{'host': ES_HOST, 'port': ES_PORT}],
        http_auth=(ES_USER, ES_PASSWORD),
        use_ssl=ES_USE_SSL,
        verify_certs=False
    )

    # job_sourcecode = {"index": ES_SOURCECODE_INDEX, "settings": body_settings_sourcecode, "prefix": "SOURCECODE__",
    #                  "date_field": "date_modified", "description": "Source Code", "module_name": "ElkEtlPythonCode",
    #                  "class_name": "ElkEtlPythonCode", "src_path": "../data"}
    
    job_tagcloud = {"index": ES_TAGCLOUD_INDEX, "settings": body_settings_generic, "prefix": "TAGCLOUD__",
                    "description": "Tag Cloud", "module_name": "ElkEtlTagCloud",
                    "class_name": "ElkEtlTagCloud", "es": es}
    
    INDEXES = {"job_tagcloud": job_tagcloud}
